<?php
	class CI_CORE_SKYPE
	{
		private $socket;
		
		function CI_CORE_SKYPE()
		{
			$this->socket=null;
			$this->timeout=10;
		}

		function open()
		{
			global $CI_CORE_CONFIG;
			
			$this->socket=socket_create(AF_UNIX, SOCK_STREAM, 0);
			if (!socket_connect($this->socket, $CI_CORE_CONFIG['path']['socks'].'/sessions-manager.sock'))
			{
				$this->close();
				return;
			}
			socket_set_nonblock($this->socket);
			socket_set_option($this->socket, SOL_SOCKET, SO_RCVTIMEO, array("sec" => 0, "usec" => 50000));
		}

		function send($command, $args=array(), $xtra_headers=array())
		{
			if (!$this->socket) return false;
			
			$args2=array();
			if (sizeof($args))
			{
				$args2['arg']=array();
				foreach ($args as $k => $v)
					array_push($args2['arg'], array('@' => array('name' => $k), '#' => $v));
			}
		
			$message=array(
				'id'			=> uniqid(date('YmdHis')),
				'type'			=> 'action',
				'action-name'	=> $command,
				'args'			=> $args2
			);

			foreach ($xtra_headers as $k => $v) $message[$k]=$v;

			$xml=ci_get_serialized_xml($message, 'message');
			$xml.="\n<stop/>\n";

			socket_send($this->socket, $xml, strlen($xml), 0);
			return $message['id'];
		}

		function ask($command, $args=array(), $xtra_headers=array())
		{
			if (!$this->socket) return false;
			
			$id=$this->send($command, $args, $xtra_headers);

			while (true)
			{
				$buffer='';
				while (true)
				{
					$read=array($this->socket);
					
					if (socket_select($read, $write = NULL, $except = NULL, $this->timeout) <= 0) return array();

					$data=socket_read($this->socket, 2048, PHP_BINARY_READ);
					if (!strlen($data)) break;

					$buffer.=$data;
					if (preg_match("/\s*<stop\s*\/>\s*/", $buffer)) break;
				}

				$buffer=preg_replace("/\s*<stop\s*\/>\s*/", '', $buffer);

				$packet=@ci_get_unserialized_xml($buffer);
				if ($packet['id'] == $id) return $packet;
			}
		}

		function close()
		{
			socket_close($this->socket);
			$this->socket=null;
		}

		function get_sessions_status()
		{
			if (!$this->socket) return false;
			
			$res=$this->ask('list-status');
			if (!$res['args']) return array();
			
			if ($res['args']['arg'][0]) $accounts=$res['args']['arg'];
			else $accounts=array($res['args']['arg']);

			if (!sizeof($accounts)) return array();

			$out=array();
			for($i=0; $i<sizeof($accounts); $i++)
			{
				$fields=split(',', $accounts[$i]['#']);

				foreach ($fields as $field)
				{
					list($k, $v)=split(':', $field, 2);
					$out[$accounts[$i]['@']['name']][$k]=$v;
				}
			}

			return $out;
		}
	}

	class CI_CORE_RESOURCES_MONITOR
	{
		private $socket;
		
		function CI_CORE_RESOURCES_MONITOR()
		{
			$this->socket=null;
		}

		function open()
		{
			global $CI_CORE_CONFIG;
			
			$this->socket=socket_create(AF_UNIX, SOCK_STREAM, 0);
			if (!socket_connect($this->socket, $CI_CORE_CONFIG['path']['socks'].'/resources-monitor.sock'))
			{
				$this->close();
				return;
			}
			socket_set_block($this->socket);
			socket_set_option($this->socket, SOL_SOCKET, SO_RCVTIMEO, array("sec" => 0, "usec" => 50000));
		}

		function get()
		{
			if (!$this->socket) return false;
			
			$message="get:all\n";
			socket_send($this->socket, $message, strlen($message), 0);

			$buffer='';
			while ($data=@socket_read($this->socket, 2048, PHP_BINARY_READ))
			{
				if ($data[0]=='.') break;
				$buffer.=$data;

				if (strlen($buffer)>8192) return "";
			}

			$out=array();
			$lines=preg_split("/[\n\r]+/", $buffer);
			foreach ($lines as $line)
			{
				list($k, $v)=split(':', $line, 2);
				if (!$k) continue;
				$out[$k]=$v;
			}

			return $out;
		}
	}

	class CI_CORE_CHANNELS_MANAGER
	{
		private $socket;
		
		function CI_CORE_CHANNELS_MANAGER()
		{
			$this->socket=null;
		}

		function open()
		{
			global $CI_CORE_CONFIG;
			
			$this->socket=socket_create(AF_UNIX, SOCK_STREAM, 0);
			if (!socket_connect($this->socket, $CI_CORE_CONFIG['path']['socks'].'/channels-manager.sock'))
			{
				$this->close();
				return;
			}
			socket_set_block($this->socket);
			socket_set_option($this->socket, SOL_SOCKET, SO_RCVTIMEO, array("sec" => 0, "usec" => 50000));
		}

		function send($module, $command, $args=array())
		{
			if (!$this->socket) return false;
			
			$args2=array();
			$args_list2=array();

			$id=uniqid('');

			$args_list=array();
			foreach ($args as $k => $v)
				array_push($args_list, "$k:\"$v\"");
		
			$message="#$id $module.$command(".join(',', $args_list).")\n";

			socket_send($this->socket, $message, strlen($message), 0);

			return $id;
		}

		function ask($module, $command, $args=array())
		{
			if (!$this->socket) return false;
			
			$id=$this->send($module, $command, $args);

			$buffer='';
			while ($data=@socket_read($this->socket, 2048, PHP_BINARY_READ))
			{
				if (preg_match("/^#(\w+) /", $data, $matches) && $matches[1] == $id)
				{
					$buffer=$data;
					break;
				}
			}

			preg_match("/^#\w+ (\w+)\.(\w+)\(([^)]+)\)$/", $buffer, $matches);
			$out['module']		= $matches[1];
			$out['command']		= $matches[2];
			$out['args']		= array();
			$args_out			= $matches[3];

			preg_match_all("/([\w+\-_]+:\"[^\"]*)\"/", $args_out, $matches);
			foreach ($matches[0] as $arg_out)
			{
				list($k, $v)=split(':', $arg_out);

				$v=preg_replace("/\"/", "", $v);
				$v=preg_replace("/\"$/", "", $v);
				
				$out['args'][$k]=$v;
			}

			return $out;
		}
	}
?>