<?php
	require_once(CI_PATH_LIBS.'/config.inc.php');
	require_once(CI_PATH_LIBS.'/skype_profiles.inc.php');
	require_once(CI_PATH_LIBS.'/skype_forwards.inc.php');
	
	class CI_SKYPE extends CI_CONFIG
	{
		function CI_SKYPE()
		{
		}

		function get_flag_align()
		{
			return ci_get_core_flag('sessions.align') ? true : false;
		}

		function set_flag_align()
		{
			ci_set_core_flag('sessions.align', 1);
		}

		function set_flag_restart($skype)
		{
			$skypes=array();
			$val=ci_get_core_flag('sessions.restart');
			if ($val) $skypes=split("\n", $val);

			if (in_array('*', $skypes)) return;
			if (in_array($skype, $skypes)) return;
			
			array_push($skypes, $skype);
			ci_set_core_flag('sessions.restart', join("\n", $skypes));
			$this->set_flag_align();
		}
		
		function get_config_file()
		{
			return CORE_PATH_BASE.'/config/skype.xml';
		}

		function get_rmap($info)
		{
			return array(
				'id'	=> $info['id'],
				'desc'	=> $info['desc'],
				'skype'	=> array(
					'username'	=> $info['skype_username'],
					'password'	=> $info['skype_password']
				),
				'vnc'	=> array(
					'screen'	=> $info['vnc_screen'],
					'password'	=> $info['vnc_password']
				),
				'enabled'	=> $info['enabled'],
				'filetransfer'	=> array(
					'enabled'			=> $info['filetransfer_enabled']
				),
				'userdata'	=> array(
					'igx-master-account'								=> $info['master_account'],
					'castiron-filetransfer-media'						=> $info['filetransfer_media'],
					'castiron-filetransfer-message-template-subject'	=> $info['filetransfer_message_template_subject'],
					'castiron-filetransfer-message-template-body'		=> $info['filetransfer_message_template_body'],
					'castiron-caller-prefix'							=> $info['caller_prefix'],
					'castiron-fallback'									=> $info['fallback']
				),
				'audio'		=> array(
					'agc'	=> $info['audio_agc'],
					'aec'	=> $info['audio_aec']
				),
				'profile'				=> $info['profile']
			);
		}

		function get_map($info)
		{
			return array(
				'id'									=> $info['id'],
				'desc'									=> $info['desc'],
				'skype_username'						=> $info['skype']['username'],
				'skype_password'						=> $info['skype']['password'],
				'vnc_screen'							=> $info['vnc']['screen'],
				'vnc_password'							=> $info['vnc']['password'],
				'enabled'								=> $info['enabled'],
				'filetransfer_enabled'					=> $info['filetransfer']['enabled'],
				'filetransfer_media'					=> $info['userdata']['castiron-filetransfer-media'],
				'filetransfer_message_template_subject'	=> $info['userdata']['castiron-filetransfer-message-template-subject'],
				'filetransfer_message_template_body'	=> $info['userdata']['castiron-filetransfer-message-template-body'],
				'master_account'						=> $info['userdata']['igx-master-account'],
				'audio_agc'								=> $info['audio']['agc'],
				'audio_aec'								=> $info['audio']['aec'],
				'profile'								=> $info['profile'],
				'caller_prefix'							=> $info['userdata']['castiron-caller-prefix'],
				'fallback'								=> $info['userdata']['castiron-fallback']
			);
		}

		function get_info($id)
		{
			$info =& parent::get_info($id);
			if (sizeof($info)) return $info;

			/* Empty (default values) */
			return array(
				'enabled'		=> 'true',
				'audio_agc'		=> 'true',
				'audio_aec'		=> 'true'
			);
		}

		function get_info_by_username($username)
		{
			$id=$this->search_username($username);
			if (!$id) return array();

			return $this->get_info($id);
		}

		function search_username($username)
		{
			$list =& $this->get_raw_list();
			for ($i=0; $i<sizeof($list); $i++)
			{
				if ($list[$i]['skype']['username'] == $username) return $list[$i]['id'];
			}

			return false;
		}

		function search_vnc($n)
		{
			$list =& $this->get_raw_list();
			for ($i=0; $i<sizeof($list); $i++)
			{
				if ($list[$i]['vnc']['screen'] == $n) return $list[$i]['id'];
			}

			return false;
		}

		function search_free_vnc()
		{
			$screens=array();
			
			$list =& $this->get_raw_list();
			for ($i=0; $i<sizeof($list); $i++)
			{
				array_push($screens, intval($list[$i]['vnc']['screen']));
			}

			sort($screens);
			for ($out=1; $out<500; $out++)
			{
				if (!in_array($out, $screens)) break;
			}

			return $out;
		}

		function task_reload($id)
		{
			$info =& parent::get_info($id);
			if (!sizeof($info)) return ci_response_ok();

			if ($info['enabled'] != 'true') return ci_response_ok();

			$this->set_flag_restart($info['skype_username']);
			return ci_response_ok();
		}

		function task_delete($id)
		{
			$this->set_flag_align();
			return parent::task_delete($id);
		}

		function task_toggle($id)
		{
			$this->set_flag_align();
			return parent::task_toggle($id);
		}

		function task_save($info)
		{
			/* Filter params */
			$info['vnc_screen']=intval($info['vnc_screen']);
			$info['skype_username']=trim(strtolower($info['skype_username']));
			
			/* New item, let's create one ID */
			if (!$info['id'])			$info['id']=md5(uniqid(''));

			/* Auto set VNC screen */
			if (!$info['vnc_screen'])	$info['vnc_screen']=$this->search_free_vnc();

			if (!$info['skype_username']) return ci_response_error(
					'field',
					array(
						'fieldname' => 'skype_username',
						'desc'		=> ci_lbl('libskype.error.invalid-username')
					)
				);

			if ($info['filetransfer_enabled'] == 'true' && !$info['filetransfer_media']) return ci_response_error(
					'field',
					array(
						'fieldname' => 'filetransfer_media',
						'desc'		=> ci_lbl('libskype.error.missing-filetransfer_media')
					)
				);

			/* Validate */
			$foo=$this->search_username($info['skype_username']);
			if ($foo && ($info['id'] != $foo))
				return ci_response_error(
					'field',
					array(
						'fieldname' => 'username',
						'desc'		=> ci_lbl('libskype.error.duplicated-username')
					)
				);

			$foo=$this->search_vnc($info['vnc_cscreen']);
			if ($foo && ($info['id'] != $foo))
				return ci_response_error(
					'field',
					array(
						'fieldname' => 'vnc_screen',
						'desc'		=> ci_lbl('libskype.error.duplicated-vnc')
					)
				);

			/* Compose fields */
			$fields=$this->get_rmap($info);

			/* Retrieve item position */
			$n=$this->get_n($info['id']);

			/* Save */
			$list =& $this->get_raw_list();
			$list[$n]=$fields;

			$this->set_flag_restart($info['skype_username']);

			$this->save_raw_config($list);
			return ci_response_ok();
		}
	}
?>
