#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time
import signal
import thread
from xml.sax import saxutils
from threading import Lock

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-1]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from s2p import *
from s2p_logger import *

sys.path.append(CORE_PATH_SDK+'/python')

from s2p_base import *

S2P_CONF_SYSTEM = xml_to_array(CORE_PATH_CONFIG+'/system.xml')
s2ps=S2P_SKYPE()

CHECK_INTERVAL	= 60
RESTART_AFTER	= 10

def sighandler_quit(signum, frame):
	s2p_log("esp", "info", "ESP stopped due to signal "+str(signum))
	s2p_deregister_pid('esp')
	
	sys.exit(0)

# List registered
def get_registered():
	global s2ps
	
	out=[]
	res=s2ps.command("", "list-registered", {})
	
	accounts=res['args']
	for k in accounts.keys(): out.append(accounts[k])
	
	return out
	
# List configured
def get_configured():
	
	S2P_CONF_SKYPE	= xml_to_array(CORE_PATH_CONFIG+'/skype.xml', 0)
	
	out=[]

	if S2P_CONF_SKYPE.has_key('item'):
		for item in S2P_CONF_SKYPE['item'].values():
			if item['+']['enabled'][0]['#'] != 'true': continue
			name=item['+']['skype'][0]['+']['username'][0]['#']

			out.append(name)
			
	return out

pid=s2p_register_pid('esp')
s2p_log("esp", "info", "ESP started with pid "+str(pid))

signal.signal(signal.SIGHUP, sighandler_quit)
signal.signal(signal.SIGINT, sighandler_quit)
signal.signal(signal.SIGQUIT, sighandler_quit)
signal.signal(signal.SIGTERM, sighandler_quit)

# Loop
accounts_status={}
while True:
	
	configured=get_configured()
	registered=get_registered()
	
	for i in configured:
		if not accounts_status.has_key(i):
			accounts_status[i]=0
		
		if i not in registered:
			accounts_status[i]=accounts_status[i]+1
		else:
			accounts_status[i]=0

	for i in accounts_status.keys():
		if not i in configured:
			del accounts_status[i]
			continue
		
		if accounts_status[i]>=RESTART_AFTER:
			s2p_log("esp", "warn", "Restart forced for "+i)
			os.system(CORE_PATH_BASE+'/bin/session.sh %s stop' % (i))
			
			home=CORE_PATH_VH+'/'+i
			os.system("rm -Rf "+home)
			
			os.system(CORE_PATH_BASE+'/bin/session.sh %s start' % (i))
			
			accounts_status[i]=0
			
		elif accounts_status[i]>=2:
			s2p_log("esp", "info", "Account "+i+" is off-line... waiting ("+str(accounts_status[i])+"/"+str(RESTART_AFTER)+")")
	
	time.sleep(CHECK_INTERVAL)

s2ps.stop()
sys.exit(0)
